@@ -1,4 +1,6 @@
-import subprocess
+import zipfile
+import io
+from typing import Tuple
 from django.db import models
 from organizations.abstract import (
     AbstractOrganization,
@@ -35,12 +37,16 @@ class NatsMessageExport(models.Model):
 
 class NatsOrganizationManager(OrgManager):
     def create_nsc(self, **kwargs):
-        from django_nats_nkeys.services import nsc_add_account
+        from django_nats_nkeys.services import nsc_add_account, nsc_jetstream_update
 
         # create django model
         org = self.create(**kwargs)
         # try create nsc account
-        return nsc_add_account(org)
+        org = nsc_add_account(org)
+        # should we enable jetstream?
+        if org.jetstream_enabled:
+            return nsc_jetstream_update(org)
+        return org
 
 
 class ActiveNatsOrganizationManager(NatsOrganizationManager, ActiveOrgManager):
@@ -66,6 +72,33 @@ class NatsOrganization(AbstractOrganization):
         NatsMessageExport, related_name="nats_organization_exports"
     )
 
+    jetstream_enabled = models.BooleanField(
+        default=False,
+        help_text="Enable JetStream for all users/apps belonging to NatsOrganization account",
+    )
+
+    jetstream_max_mem = models.CharField(
+        default="1M",
+        max_length=32,
+        help_text="JetStream memory resource limits (shared across all users/apps beloning to NatsOrganization account)",
+    )
+
+    jetstream_max_file = models.CharField(
+        default="5M",
+        max_length=32,
+        help_text="JetStream file resource limits (shared across all users/apps beloning to NatsOrganization account)",
+    )
+
+    jetstream_max_streams = models.PositiveIntegerField(
+        default=10,
+        help_text="JetStream max number of streams (shared across all users/apps beloning to NatsOrganization account)",
+    )
+
+    jetstream_max_consumers = models.PositiveIntegerField(
+        default=10,
+        help_text="JetStream max number of consumers (shared across all users/apps beloning to NatsOrganization account)",
+    )
+
     def nsc_validate(self):
         from .services import nsc_validate
 
@@ -86,6 +119,8 @@ class AbstractNatsApp(models.Model):
         max_length=255, help_text="Output of `nsc describe account`", default=dict
     )
 
+    bearer = models.BooleanField(default=False)
+
     allow_pub = models.CharField(
         max_length=255,
         null=True,
@@ -143,7 +178,7 @@ class NatsOrganizationUser(AbstractOrganizationUser, AbstractNatsApp):
         return nsc_validate(account_name=self.organization.name)
 
 
-class NatsOrganizationApp(AbstractNatsApp):
+class AbstractNatsOrganizationApp(AbstractNatsApp):
     """
     Corresponds to a NATS user/client within an Account group
     https://docs.nats.io/running-a-nats-service/configuration/securing_nats/accounts
@@ -151,7 +186,11 @@ class NatsOrganizationApp(AbstractNatsApp):
 
     objects = NatsOrganizationAppManager()
 
+    ORGANIZATION_USER_RELATED_NAME = "nats_organization_apps"
+    ORGANIZATION_RELATED_NAME = "nats_organization_apps"
+
     class Meta:
+        abstract = True
         constraints = [
             models.UniqueConstraint(
                 fields=["app_name", "organization_user"],
@@ -160,20 +199,72 @@ class NatsOrganizationApp(AbstractNatsApp):
         ]
 
     organization_user = models.ForeignKey(
-        NatsOrganizationUser, on_delete=models.CASCADE, related_name="nats_apps"
+        NatsOrganizationUser,
+        on_delete=models.CASCADE,
+        related_name=ORGANIZATION_USER_RELATED_NAME,
     )
     organization = models.ForeignKey(
         NatsOrganization,
         on_delete=models.CASCADE,
-        related_name="nats_apps",
+        related_name=ORGANIZATION_RELATED_NAME,
     )
 
+    def generate_jwt(self) -> str:
+        """
+        Extracts base64-encoded JWT string from nkey credential
+        Intended for use as bearer auth token in MQTT client
+        """
+        creds = self.generate_creds()
+        parts = creds.split("\n")
+        if len(parts) > 2:
+            return creds.split("\n")[1]
+        return creds
+
+    def generate_creds(self) -> str:
+        """
+        Generates full credential will format:
+
+        -----BEGIN NATS USER JWT-----
+        <jwt>
+        ------END NATS USER JWT------
+
+        ************************* IMPORTANT *************************
+        NKEY Seed printed below can be used to sign and prove identity.
+        NKEYs are sensitive and should be treated as secrets.
+
+        -----BEGIN USER NKEY SEED-----
+        <seed>
+        ------END USER NKEY SEED------
+
+        *************************************************************
+
+        """
+        from django_nats_nkeys.services import nsc_generate_creds
+
+        return nsc_generate_creds(self.organization.name, self.app_name)
+
+    def generate_creds_zip(self, filename="nats.creds.zip") -> Tuple[str, bytes]:
+        """
+        Returns a Tuple of (filename, compressed bytes)
+        """
+        creds = self.generate_creds()
+        # do not write sensitive credentials to disk
+        # instead, write to memory buffer
+        zip_buffer = io.BytesIO()
+        with zipfile.ZipFile(zip_buffer, "a") as zip_obj:
+            zip_obj.writestr(filename, creds)
+        return filename, zip_buffer.getvalue()
+
     def nsc_validate(self):
         from .services import nsc_validate
 
         return nsc_validate(account_name=self.organization.name)
 
 
+class NatsOrganizationApp(AbstractNatsOrganizationApp):
+    pass
+
+
 class NatsOrganizationOwner(AbstractOrganizationOwner):
     """Identifies ONE user, by AccountUser, to be the owner"""
 
